DROP TABLE  T_EMPLOYEE_MASTER ;
DROP TABLE  T_PALLET_PRODUCT ;
DROP TABLE T_REVIEW_ERROR_TYPE;      
DROP TABLE  T_PALLET_MASTER ;
DROP TABLE  T_COMPANY_LOCATION ;
DROP TABLE T_BAY_MASTER;
DROP TABLE T_LOAD_MASTER;
DROP TABLE  T_LOCATION_MASTER ;
DROP TABLE  T_TYPE_CONFERENCE ;
DROP TABLE  T_COMPANY_MASTER ;

CREATE  TABLE  T_COMPANY_MASTER (
  COMPANY_ID NVARCHAR(20) NOT NULL ,
  COMPANY_DESCRIPTION NVARCHAR(50) NOT NULL ,
  COMPANY_ADDRESS NVARCHAR(200) NULL ,
  PRIMARY KEY (COMPANY_ID) );

CREATE  TABLE  T_LOCATION_MASTER (
  LOCATION_ID NVARCHAR(20) NOT NULL ,
  LOCATION_DESCRIPTION NVARCHAR(50) NOT NULL ,
  PRIMARY KEY (LOCATION_ID) );


CREATE  TABLE  T_EMPLOYEE_MASTER (
  EMPLOYEE_ID NVARCHAR(20) NOT NULL ,
  EMPLOYEE_NAME NVARCHAR(50) NOT NULL ,
  EMPLOYEE_PASSWORD NVARCHAR(45) NOT NULL ,
  COMPANY_ID NVARCHAR(20) NOT NULL ,
  LOCATION_ID NVARCHAR(20) NOT NULL ,
  PRIMARY KEY (EMPLOYEE_ID) ,
  CONSTRAINT fk_Login_COMPANY1
    FOREIGN KEY (COMPANY_ID )
    REFERENCES T_COMPANY_MASTER (COMPANY_ID )
    ON DELETE NO ACTION
    ON UPDATE CASCADE,
  CONSTRAINT fk_Login_LOCATION1
    FOREIGN KEY (LOCATION_ID )
    REFERENCES T_LOCATION_MASTER (LOCATION_ID )
    ON DELETE NO ACTION
    ON UPDATE CASCADE);


CREATE  TABLE  T_TYPE_CONFERENCE (
  TYPE_CONFERENCE_ID INT NOT NULL ,
  TYPE_CONFERENCE_DESCRIPTION NVARCHAR(45) NULL ,
  PRIMARY KEY (TYPE_CONFERENCE_ID) );

CREATE  TABLE  T_COMPANY_LOCATION (
  COMPANY_ID NVARCHAR(20) NOT NULL ,
  LOCATION_ID NVARCHAR(20) NOT NULL ,
  TYPE_CONFERENCE_ID INT NOT NULL ,
  PRIMARY KEY (COMPANY_ID, LOCATION_ID) ,
  CONSTRAINT fk_CompanyLocation_COMPANY1
    FOREIGN KEY (COMPANY_ID )
    REFERENCES T_COMPANY_MASTER (COMPANY_ID )
    ON DELETE NO ACTION
    ON UPDATE CASCADE,
  CONSTRAINT fk_CompanyLocation_LOCATION1
    FOREIGN KEY (LOCATION_ID )
    REFERENCES T_LOCATION_MASTER (LOCATION_ID )
    ON DELETE NO ACTION
    ON UPDATE CASCADE,
  CONSTRAINT fk_CompanyLocation_TYPE_CONFERENCE1
    FOREIGN KEY (TYPE_CONFERENCE_ID )
    REFERENCES T_TYPE_CONFERENCE (TYPE_CONFERENCE_ID )
    ON DELETE NO ACTION
    ON UPDATE CASCADE);

CREATE  TABLE T_PALLET_MASTER (
  BAY_ID NVARCHAR(20) NOT NULL ,
  TRIP_NUMBER INT NOT NULL ,
  COMPANY_ID NVARCHAR(20) NULL ,
  LOCATION_ID NVARCHAR(20) NULL ,
  PALLET_DATE_DELIVERY DATETIME NULL ,
  PALLET_REVIEWED BIT NOT NULL ,
  NUMBER_OF_SKUS INT NOT NULL ,
  START_TIME DATETIME NULL ,
  END_TIME DATETIME NULL ,
  PRIMARY KEY (BAY_ID, TRIP_NUMBER) ,
  CONSTRAINT fk_PALLET_CompanyLocation1
    FOREIGN KEY (COMPANY_ID , LOCATION_ID )
    REFERENCES T_COMPANY_LOCATION (COMPANY_ID , LOCATION_ID )
    ON DELETE NO ACTION
    ON UPDATE CASCADE) ;


CREATE  TABLE  T_REVIEW_ERROR_TYPE (
  ERROR_ID NVARCHAR(20) NOT NULL ,
  ERROR_DESCRIPTION NVARCHAR(45) NOT NULL ,
  PRIMARY KEY (ERROR_ID) );

  
  CREATE  TABLE  T_BAY_MASTER (
  BAY_ID NVARCHAR(20) NOT NULL ,
  BAY_LABEL NVARCHAR(5) NULL ,
  COMPANY_ID NVARCHAR(20) NOT NULL ,
  LOCATION_ID NVARCHAR(20) NOT NULL ,
  PRIMARY KEY (BAY_ID, COMPANY_ID, LOCATION_ID) ,
  CONSTRAINT fk_T_BAY_MASTER_T_COMPANY_MASTER1
    FOREIGN KEY (COMPANY_ID )
    REFERENCES T_COMPANY_MASTER (COMPANY_ID )
    ON DELETE NO ACTION
    ON UPDATE CASCADE,
  CONSTRAINT fk_T_BAY_MASTER_T_LOCATION_MASTER1
    FOREIGN KEY (LOCATION_ID )
    REFERENCES T_LOCATION_MASTER (LOCATION_ID )
    ON DELETE NO ACTION
    ON UPDATE CASCADE);
  
CREATE  TABLE T_PALLET_PRODUCT (
  TRIP_NUMBER INT NOT NULL ,
  PRODUCT_ID NVARCHAR(20) NOT NULL ,
  BAY_ID NVARCHAR(20) NOT NULL ,
  QUANTITY INT NOT NULL ,
  ERROR_ID NVARCHAR(20) NULL ,
  COMPANY_ID NVARCHAR(20) NOT NULL ,
  LOCATION_ID NVARCHAR(20) NOT NULL ,
  PRODUCT_DESCRIPTION_LONG NVARCHAR(50) NULL ,
  PRODUCT_DESCRIPTION_SHORT NVARCHAR(10) NULL ,
  PICKING_QUANTITY_DESCRIPTION NVARCHAR(50) NULL ,
  PRIMARY KEY (PRODUCT_ID, BAY_ID, TRIP_NUMBER) ,
  CONSTRAINT fk_T_PALLET_PRODUCT_T_ERROR_TYPE1
    FOREIGN KEY (ERROR_ID )
    REFERENCES T_REVIEW_ERROR_TYPE (ERROR_ID )
    ON DELETE NO ACTION
    ON UPDATE NO ACTION,
  CONSTRAINT fk_T_PALLET_PRODUCT_T_PALLET_MASTER1
    FOREIGN KEY (BAY_ID , TRIP_NUMBER )
    REFERENCES T_PALLET_MASTER (BAY_ID , TRIP_NUMBER )
    ON DELETE NO ACTION
    ON UPDATE NO ACTION,
  CONSTRAINT fk_T_PALLET_PRODUCT_T_COMPANY_LOCATION1
    FOREIGN KEY (COMPANY_ID , LOCATION_ID )
    REFERENCES T_COMPANY_LOCATION (COMPANY_ID , LOCATION_ID )
    ON DELETE NO ACTION
    ON UPDATE NO ACTION);
	
	

	
	

CREATE TABLE T_LOAD_MASTER (
  COMPANY_ID NVARCHAR(20) NOT NULL ,
  LOCATION_ID NVARCHAR(20) NOT NULL ,
  VEHICLE_ID NVARCHAR(20) NULL ,
  VEHICLE_DESCRIPTION NVARCHAR(50) NULL ,
  TRIP_NUMBER INT NOT NULL,
  PRIMARY KEY (COMPANY_ID, LOCATION_ID, TRIP_NUMBER),
  CONSTRAINT fk_T_VEHICLE_MASTER_T_COMPANY_MASTER1
    FOREIGN KEY (COMPANY_ID )
    REFERENCES T_COMPANY_MASTER (COMPANY_ID )
    ON DELETE NO ACTION
    ON UPDATE CASCADE,
  CONSTRAINT fk_T_VEHICLE_MASTER_T_LOCATION_MASTER1
    FOREIGN KEY (LOCATION_ID )
    REFERENCES T_LOCATION_MASTER (LOCATION_ID )
    ON DELETE NO ACTION
    ON UPDATE CASCADE);
	
	



--Devem ser preenchidos antes do cadastro dos pallets.
INSERT INTO T_COMPANY_MASTER VALUES('1', 'SPAIPA', 'COMPANY_ADDRESS');
INSERT INTO T_LOCATION_MASTER VALUES('CTA', 'LOCATION_DESCRIPTION');
INSERT INTO T_TYPE_CONFERENCE VALUES(2, 'LIVRE'); 
INSERT INTO T_EMPLOYEE_MASTER VALUES('EMPSPAIPA', 'EMPSPAIPA_NAME', 'EMPPASS', '1', 'CTA'); 
INSERT INTO T_COMPANY_LOCATION VALUES('1', 'CTA', 2);

INSERT INTO T_BAY_MASTER VALUES('25', 'baia 25', '1', 'CTA');





INSERT INTO T_REVIEW_ERROR_TYPE VALUES(1, 'MONTAGEM');
INSERT INTO T_REVIEW_ERROR_TYPE VALUES(2, 'FALTA');
INSERT INTO T_REVIEW_ERROR_TYPE VALUES(3, 'SOBRA');
INSERT INTO T_REVIEW_ERROR_TYPE VALUES(4, 'INVERSAO');



