﻿#region

using System;
using System.Collections;
using System.Collections.Generic;
using System.Data;
using System.Windows.Forms;
using LoadReview.Grid;
using LoadReview.Persistencia.Pojo;
using LoadReviewLibrary.Persistencia.Pojo;
using LoadReviewLibrary.Persistencia.Pojo.Generics;

#endregion

namespace LoadReview
{
    public partial class Check : Form
    {
        private const int ERROR_ID_SALVAR_PALLET = 6;

        private const String ERROR_MSG_SALVAR_PALLET =
            "Não foi possivel salvar os dados. Verifique conexão com banco de dados";

        private const String MSG_PALLET_ARMAZENADO_SUCESSO = "Pallet Conferido com sucesso";

        private const String MSG_PALLET_CONFERIDO =
            "Informações não poderão ser alteradas posteriormente. Deseja finalizar a conferência do Pallet?";

        private const String MSG_TOTAL_PALLET_INCORRETO = "Total Conferido diferente do Total do Pallet";

        private CompanyLocation companyLocation;


        private PalletPDA pallet;
        private int totalConferido;

        public Check()
        {
            InitializeComponent();
        }

        public Check(IPallet pallet,
                     CompanyLocation companyLocation) : this()
        {
            this.pallet = (PalletPDA) pallet;
            this.companyLocation = companyLocation;

            InitLabels();
            MontaGrid();
        }

        protected DataGridCustomTextBoxColumn colunaDescription { get; set; }
        protected DataGridCustomTextBoxColumn colunaPicking { get; set; }
        protected DataGridCustomTextBoxColumn ColunaQuantity { get; set; }

        protected DataTable Tabela { get; set; }

        public int TotalConferido
        {
            get { return totalConferido; }

            set
            {
                totalConferido = value;
                lblTotal.Text = totalConferido.ToString();
            }
        }


        private void InitLabels()
        {
            lblDeliveryDate.Text = pallet.DeliveryDate.ToString();
            lblTripNumber.Text = pallet.TripNumber.ToString();
            lblVehicle.Text = pallet.Vehicle.Id;
            lblBay.Text = pallet.Bay.Label;
            lblTotal.Text = TotalConferido.ToString();
        }


        private void MontaGrid()
        {
            //cria dataTable
            Tabela = new DataTable("Conferência");

            //adiciona as colunas ao DataTable
            Tabela.Columns.Add("Description");
            Tabela.Columns.Add("Quantity");
            Tabela.Columns.Add("Picking");

            //cria as colunas
            colunaDescription = new DataGridCustomTextBoxColumn();
            ColunaQuantity = new DataGridCustomTextBoxColumn();
            colunaPicking = new DataGridCustomTextBoxColumn();

            //associa as colunas ao Grid de Conferencia
            colunaDescription.Owner = grdConferencia;
            ColunaQuantity.Owner = grdConferencia;
            colunaPicking.Owner = grdConferencia;


            //faz o mapeamento das colunas
            colunaDescription.MappingName = "DescriptionLong";
            colunaPicking.MappingName = "Picking";
            ColunaQuantity.MappingName = "Mask";

            //define o Titulo da coluna
            colunaDescription.HeaderText = "DESCRIPTION";
            colunaPicking.HeaderText = "PQ";
            ColunaQuantity.HeaderText = "QT";


            //define tamano das colunas
            colunaDescription.Width = 165;
            ColunaQuantity.Width = 29;
            colunaPicking.Width = 35;


            //desabilita a edição das colunas Description e Picking.
            colunaDescription.ReadOnly = true;
            colunaPicking.ReadOnly = true;
            ColunaQuantity.ReadOnly = false;

            //Adiciona as colunas ao TableStyle
            dataGridTableStyleConferencia.GridColumnStyles.Add(colunaDescription);
            dataGridTableStyleConferencia.GridColumnStyles.Add(ColunaQuantity);
            dataGridTableStyleConferencia.GridColumnStyles.Add(colunaPicking);

            ///////////////////////


            dataGridTableStyleConferencia.MappingName = "ArrayList";
            grdConferencia.DataSource = new ArrayList(pallet.Products);
        }

        /// <summary>
        /// Adiciona Eventos para cada produto
        /// </summary>
        /// <param name="products">Listra de produtos do Pallet.</param>
        protected virtual void AddEventosProduct(List<Product> products)
        {
            for (int i = 0; i < products.Count; i++)
            {
                ProductPDA product = (ProductPDA) products[i];
                product.SelectError += ProductSelectError;
                product.IncrementTot += IncrementTotal;

                product.MontaMascara += montaMascara;
            }
        }

        protected virtual String montaMascara(int value)
        {
            throw new Exception("Implementar método monta Mascara");
        }


        public int ProductSelectError()
        {
            FormError form = new FormError();
            form.ShowDialog();

            // return form.ErrorId;
            return form.ErrorId;
        }

        protected void IncrementTotal(String value)
        {
            TotalConferido = TotalConferido + int.Parse(value);
        }

        protected void ReduzirTotal(String value)
        {
            TotalConferido = TotalConferido - int.Parse(value);
        }

        private void menuItem1_Click(object sender, EventArgs e)
        {
            ReadPallet idpallet = new ReadPallet(companyLocation);
            idpallet.Show();
            Close();
        }

        private void menuItem2_Click(object sender, EventArgs e)
        {
            if (TotalConferido != pallet.Total)
            {
                MessageBox.Show(MSG_TOTAL_PALLET_INCORRETO, "Error",
                                MessageBoxButtons.OK,
                                MessageBoxIcon.Exclamation,
                                MessageBoxDefaultButton.Button3);
            }
            else
            {
                DialogResult result = MessageBox.Show(
                    MSG_PALLET_CONFERIDO,
                    "Finalizar", MessageBoxButtons.YesNo,
                    MessageBoxIcon.Question,
                    MessageBoxDefaultButton.Button3);

                ///Verifica se usuário confirmou a operação.
                if (result == DialogResult.Yes)
                {
                    pallet.Reviewed = true;
                    if (pallet.Update())

                    {
                        ReadPallet idpallet = new ReadPallet(companyLocation);
                        idpallet.Show();

                        DialogResult sucesso = MessageBox.Show(
                            MSG_PALLET_ARMAZENADO_SUCESSO,
                            "Finalizar", MessageBoxButtons.OK,
                            MessageBoxIcon.None,
                            MessageBoxDefaultButton.Button3);

                        Close();
                    }

                    else
                    {
                        MessageBox.Show(ERROR_MSG_SALVAR_PALLET, "Erro ao salvar dados",
                                        MessageBoxButtons.OK,
                                        MessageBoxIcon.Asterisk,
                                        MessageBoxDefaultButton.Button3);
                    }
                }
            }
        }

        private void label5_ParentChanged(object sender, EventArgs e)
        {
        }
    }
}