﻿#region

using System;
using System.Collections.Generic;
using System.Drawing;
using LoadReview.Grid;
using LoadReview.Persistencia.Pojo;
using LoadReviewLibrary.Persistencia.Pojo;
using LoadReviewLibrary.Persistencia.Pojo.Generics;
using Microsoft.WindowsCE.Forms;

#endregion

namespace LoadReview
{
    internal class CheckFree : Check
    {
        private DataGridCustomColumnButtonGreen colunaConferido = new DataGridCustomColumnButtonGreen();
        private DataGridCustomColumnButtonRed colunaErro = new DataGridCustomColumnButtonRed();


        public CheckFree(IPallet pallet,
                         CompanyLocation companyLocation)
            : base(pallet, companyLocation)
        {
            colunaErro.OnAlteraEstado += ProductSelectError;

            colunaConferido.OnAlteraEstadoTrue +=
                IncrementTotal;
            colunaConferido.OnAlteraEstadoFalse +=
                ReduzirTotal;


            AddEventosProduct(pallet.Products);


            MontaGrid();

            SystemSettings.ScreenOrientation =
                ScreenOrientation.Angle270;
        }


        /// <summary>
        /// Monta grid personalizado para conferência livre.
        /// /tamanho das colunas são definidos com números absolutos.
        ///
        /// </summary>
        private void MontaGrid()
        {
            grdConferencia.Click += Update;

            Tabela.Columns.Add("Conferido");
            Tabela.Columns.Add("Erro");
            ColunaQuantity.ReadOnly = true;

            colunaConferido.Owner = grdConferencia;
            colunaErro.Owner = grdConferencia;

            colunaConferido.MappingName = "Verify";
            colunaErro.MappingName = "ErrorType";

            colunaErro.Width = grdConferencia.Width*9/100 + 10;
            colunaErro.CorOn = Color.FromArgb(255, 0, 0);
            colunaErro.CorOut = Color.FromArgb(140, 0, 0);

            colunaConferido.Width = grdConferencia.Width*9/100 + 10;
            colunaConferido.CorOn = Color.FromArgb(0, 255, 0);
            colunaConferido.CorOut = Color.FromArgb(0, 65, 0);

            dataGridTableStyleConferencia.GridColumnStyles.Add(colunaConferido);
            dataGridTableStyleConferencia.GridColumnStyles.Add(colunaErro);
        }

        /// <summary>
        /// Adiciona Eventos para cada produto
        /// </summary>
        /// <param name="products">Lista de produtos do Pallet</param>
        protected override void AddEventosProduct(List<Product> products)
        {
            base.AddEventosProduct(products);

            for (int i = 0; i < products.Count; i++)
            {
                ProductPDA product = (ProductPDA) products[i];
                product.MontaMascara += montaMascara;
            }
        }


        /// <summary>
        /// Executado sempre que o grid sofre alguma modificação.
        /// </summary>
        /// <param name="obj"></param>
        /// <param name="ev"></param>
        private void Update(Object obj, EventArgs ev)
        {
            //chama métodos para alterar estado dos Button.
            colunaErro.AlteraEstadoButton();
            colunaConferido.AlteraEstadoButton();

            //redesenha o grid
            grdConferencia.Invalidate();
        }


        /// <summary>
        /// Monta a mascara para a coluna quantidade.
        /// No caso de conferência livre não há mascara. 
        /// Retorna a string completa.
        /// </summary>
        /// <param name="value">Pode ser passado qualquer valor, o resultado será sempre o mesmo.</param>
        /// <returns>String completa com a quantidade de cada produto. Como o tipo de conferência é livre, não esconde nenhum digito. </returns>
        protected override String montaMascara(int value)
        {
            return Convert.ToString(value);
        }
    }
}