﻿#region

using System;
using System.Collections.Generic;
using System.Data.SqlServerCe;
using System.Windows.Forms;
using LoadReview.Persistencia.Pojo;
using LoadReviewLibrary.Persistencia.Dao.Generics;
using LoadReviewLibrary.Persistencia.Pojo;
using LoadReviewLibrary.Persistencia.Pojo.Generics;

#endregion

namespace LoadReview.Persistencia.Dao
{
    internal class PalletDaoPDA : IPalletDao
    {
        private const int ERROR_ID_INSERT_PRODUCT = 7;

        private const int ERROR_ID_UPDATE_PRODUCT = 8;
        private const String ERROR_MSG_INSERT_PRODUCT = "Não foi possivel inserir o seguinte produto:";
        private const String ERROR_MSG_UPDATE_PRODUCT = "Não foi possivel atualizar o seguinte produto:";

        private const String SQL_INSERT_PALLET =
            "INSERT INTO T_PALLET_MASTER(BAY_ID, TRIP_NUMBER, COMPANY_ID, LOCATION_ID, PALLET_DATE_DELIVERY, PALLET_REVIEWED, NUMBER_OF_SKUS, START_TIME, END_TIME) VALUES(@bay, @trip, @company, @location, @delivery, @reviewed, @skus, @start, @end)";

        private const String SQL_SELECT_PALLET =
            "SELECT * FROM T_PALLET_MASTER WHERE COMPANY_ID = @Company AND LOCATION_ID = @Location AND BAY_ID = @Bay AND TRIP_NUMBER = @trip";

        private const String SQL_UPDATE_PALLET =
            "UPDATE T_PALLET_MASTER SET PALLET_DATE_DELIVERY = @date, PALLET_REVIEWED = @Verify, NUMBER_OF_SKUS = @skus, START_TIME = @start, END_TIME = @end WHERE BAY_ID = @bay AND TRIP_NUMBER = @trip AND COMPANY_ID = @company AND LOCATION_ID = @location";


        private SqlCeCommand cmdInsert;
        private SqlCeCommand cmdUpdate;
        private List<SqlCeParameter> parameters;
        private IProductDao productDao;


        public PalletDaoPDA()
        {
            connectionPDA = new ConnectionPDA();
            connectionPDA.Open();
            productDao = DaoFactory.CreateProductDao();

            cmdInsert = new SqlCeCommand(SQL_INSERT_PALLET, connectionPDA.connection);
            cmdInsert.Prepare();
            cmdUpdate = new SqlCeCommand(SQL_UPDATE_PALLET, connectionPDA.connection);
            cmdUpdate.Prepare();

            parameters = new List<SqlCeParameter>();
        }

        public ConnectionPDA connectionPDA { get; set; }

        #region IPalletDao Members

        /// <summary>
        /// Retorna um Pallet.
        /// </summary>
        /// <param name="cp">CompanyLocation do Pallet a ser retornado.</param>
        /// <param name="bay">BayId do Pallet.</param>
        /// <param name="tripNumber">TripNumber do Pallet.</param>
        /// <returns>Pallet consultado. Retorna null se não for achado nenhum pallet.</returns>
        public IPallet Select(CompanyLocation cp, String bay, int tripNumber)
        {
            try
            {
                //cria os parametros da consulta SQL
                SqlCeParameter company = new SqlCeParameter();
                SqlCeParameter location = new SqlCeParameter();
                SqlCeParameter bayid = new SqlCeParameter();
                SqlCeParameter trip = new SqlCeParameter();

                //cria lista para guardar os parametros da consulta SQL
                List<SqlCeParameter> parameters = new List<SqlCeParameter>();

                //define propriedades do parametro @Company
                company.ParameterName = "Company";
                company.Value = cp.Company.Id;


                //define propriedades do parametro @Location
                location.ParameterName = "Location";
                location.Value = cp.Location.Id;


                //define propriedades do parametro @Bay
                bayid.ParameterName = "Bay";
                bayid.Value = bay;


                //define propriedades do parametro @trip
                trip.ParameterName = "trip";
                trip.Value = tripNumber;


                //adiciona os parametros à lista
                parameters.Add(company);
                parameters.Add(location);
                parameters.Add(bayid);
                parameters.Add(trip);

                SqlCeDataReader reader = connectionPDA.Select(SQL_SELECT_PALLET, parameters);

                IBayDao bayDao = DaoFactory.CreateBayDao();
                IVehicleDao vehicleDao = DaoFactory.CreateVehicleDao();

                while (reader.Read())
                {
                   
               

                    Bay baia = bayDao.Select(bay, cp.Company.Id, cp.Location.Id);
                    Vehicle vehicle = vehicleDao.Select(cp.Company.Id, cp.Location.Id, tripNumber);

                    PalletPDA pallet = new PalletPDA(baia, tripNumber);
                    pallet.Vehicle = vehicle;

                    pallet.Products = productDao.ListProduct(baia.Id, pallet.TripNumber, cp);
                    pallet.Skus = reader.GetInt32(reader.GetOrdinal("NUMBER_OF_SKUS"));

                    pallet.Reviewed = reader.GetBoolean(reader.GetOrdinal("PALLET_REVIEWED"));
                    return pallet;
                }
                return null;
            }

            finally
            {
                connectionPDA.Close();
            }
        }

        public Boolean Update(IPallet plt)
        {
            Pallet pallet = (Pallet) plt;

            try
            {

                SqlCeParameter delivery = new SqlCeParameter();
                SqlCeParameter pVerify = new SqlCeParameter();
                SqlCeParameter skus = new SqlCeParameter();
                SqlCeParameter start = new SqlCeParameter();
                SqlCeParameter end = new SqlCeParameter();
                SqlCeParameter bay = new SqlCeParameter();
                SqlCeParameter trip = new SqlCeParameter();
                SqlCeParameter company = new SqlCeParameter();
                SqlCeParameter location = new SqlCeParameter();


                List<SqlCeParameter> parameters = new List<SqlCeParameter>();

                delivery.ParameterName = "date";
                if (pallet.DeliveryDate == null)
                    delivery.Value = DBNull.Value;
                else
                    delivery.Value = pallet.DeliveryDate;

                pVerify.ParameterName = "Verify";
                pVerify.Value = pallet.Reviewed;

                skus.ParameterName = "skus";
                skus.Value = pallet.Skus;

                start.ParameterName = "start";
                if (pallet.StartTime == null)
                    start.Value = DBNull.Value;
                else
                    start.Value = pallet.StartTime;

                end.ParameterName = "end";
                if (pallet.EndTime == null)
                    end.Value = DBNull.Value;
                else
                    end.Value = pallet.EndTime;

                bay.ParameterName = "bay";
                bay.Value = pallet.Bay.Id;

                trip.ParameterName = "trip";
                trip.Value = pallet.TripNumber;

                company.ParameterName = "company";
                company.Value = pallet.CompanyLocation.Company.Id;

                location.ParameterName = "location";
                location.Value = pallet.CompanyLocation.Location.Id;

                parameters.Add(delivery);
                parameters.Add(pVerify);
                parameters.Add(skus);
                parameters.Add(start);
                parameters.Add(end);
                parameters.Add(bay);
                parameters.Add(trip);
                parameters.Add(company);
                parameters.Add(location);

                connectionPDA.InitTransaction();


                connectionPDA.ExecutePreparedCommandSql(cmdUpdate, parameters);

                foreach (Product product in pallet.Products)
                {
                    if (!productDao.Update(product, pallet))
                    {
                        MessageBox.Show(
                            ERROR_MSG_INSERT_PRODUCT + "Pallet: " + pallet.TripNumber.ToString() + pallet.Bay +
                            "\n Product: " + product.Id + "\n Company: " + pallet.CompanyLocation.Company.Id +
                            "\n Location: " + pallet.CompanyLocation.Location.Id, "Update Error", MessageBoxButtons.OK,
                            MessageBoxIcon.Asterisk, MessageBoxDefaultButton.Button3);

                        connectionPDA.EndTransaction(false);
                        return false;
                    }
                }


                connectionPDA.EndTransaction(true);
                return true;
            }
            catch (Exception e)
            {
                connectionPDA.EndTransaction(false);
                return false;
            }


//marcação de ignore
#pragma warning disable 162
            return false;
#pragma warning restore 162
        }

        public bool Insert(IPallet plt)
        {
            Pallet pallet = (Pallet) plt;
            

            try
            {
                //cria os parametros
                SqlCeParameter bay = new SqlCeParameter();
                SqlCeParameter trip = new SqlCeParameter();
                SqlCeParameter company = new SqlCeParameter();
                SqlCeParameter location = new SqlCeParameter();
                SqlCeParameter dateDelivery = new SqlCeParameter();
                SqlCeParameter reviewed = new SqlCeParameter();
                SqlCeParameter skus = new SqlCeParameter();
                SqlCeParameter startTime = new SqlCeParameter();
                SqlCeParameter endTime = new SqlCeParameter();

                //define propriedades dos parametros.
                bay.ParameterName = "bay";
                bay.Value = pallet.Bay.Id;

                trip.ParameterName = "trip";
                trip.Value = pallet.TripNumber;

                company.ParameterName = "company";
                company.Value = pallet.CompanyLocation.Company.Id;

                location.ParameterName = "location";
                location.Value = pallet.CompanyLocation.Location.Id;

                dateDelivery.ParameterName = "delivery";
                if (pallet.DeliveryDate == null)
                    dateDelivery.Value = DBNull.Value;
                else
                    dateDelivery.Value = pallet.DeliveryDate;

                reviewed.ParameterName = "reviewed";
                reviewed.Value = pallet.Reviewed;

                skus.ParameterName = "skus";
                skus.Value = pallet.Skus;

                startTime.ParameterName = "start";
                if (pallet.StartTime == null)
                    startTime.Value = DBNull.Value;
                else
                    startTime.Value = pallet.StartTime;

                endTime.ParameterName = "end";
                if (pallet.EndTime == null)
                    endTime.Value = DBNull.Value;
                else
                    endTime.Value = pallet.EndTime;

                //cria a Lista e adiciona os parametros
                List<SqlCeParameter> list = new List<SqlCeParameter>();
                list.Add(bay);
                list.Add(trip);
                list.Add(company);
                list.Add(location);
                list.Add(dateDelivery);
                list.Add(reviewed);
                list.Add(skus);
                list.Add(startTime);
                list.Add(endTime);

                connectionPDA.InitTransaction();


                connectionPDA.ExecutePreparedCommandSql(cmdUpdate, list);

                foreach (Product product in pallet.Products)
                {
                    if (!productDao.Insert(product, pallet))
                    {
                        MessageBox.Show(
                            ERROR_MSG_INSERT_PRODUCT + "Pallet: " + pallet.TripNumber.ToString() + pallet.Bay +
                            "\n Product: " + product.Id + "\n Company: " + pallet.CompanyLocation.Company.Id +
                            "\n Location: " + pallet.CompanyLocation.Location.Id, "Insert Error", MessageBoxButtons.OK,
                            MessageBoxIcon.Asterisk, MessageBoxDefaultButton.Button3);

                        connectionPDA.EndTransaction(false);
                        return false;
                    }
                }


                connectionPDA.EndTransaction(true);
                return true;
            }

            catch (Exception e)
            {
                connectionPDA.EndTransaction(false);
                return false;
            }
        }

        public List<Pallet> PalletList(CompanyLocation cp, Boolean review)
        {
            throw new Exception("Método não implementado");
        }

        #endregion
    }
}