﻿#region

using System;
using System.Collections.Generic;
using System.Data;
using System.Data.SqlServerCe;
using LoadReview.Persistencia.Pojo;
using LoadReviewLibrary.Persistencia.Dao.Generics;
using LoadReviewLibrary.Persistencia.Pojo;
using LoadReviewLibrary.Persistencia.Pojo.Generics;

#endregion

namespace LoadReview.Persistencia.Dao
{
    internal class ProductDaoPDA : IProductDao
    {
        private const String SQL_INSERT_PRODUCT =
            "INSERT INTO T_PALLET_PRODUCT(TRIP_NUMBER, PRODUCT_ID, BAY_ID, QUANTITY, ERROR_ID, COMPANY_ID, LOCATION_ID, PRODUCT_DESCRIPTION_LONG, PRODUCT_DESCRIPTION_SHORT, PICKING_QUANTITY_DESCRIPTION ) VALUES (@trip, @productId, @bay, @quantity, @error, @company, @location, @descLong, @descShort, @picking)";

        private const String SQL_SELECT_PRODUCT =
            "SELECT * FROM T_PALLET_PRODUCT pallet WHERE pallet.BAY_ID = @bayid AND pallet.TRIP_NUMBER = @trip AND pallet.COMPANY_ID = @company AND pallet.LOCATION_ID = @location";

        private const String SQL_UPDATE_PALLET =
            "UPDATE T_PALLET_PRODUCT SET QUANTITY = @quantity, ERROR_ID = @iderror, PRODUCT_DESCRIPTION_LONG = @descLong, PRODUCT_DESCRIPTION_SHORT = @descShort, PICKING_QUANTITY_DESCRIPTION = @picking  WHERE PRODUCT_ID = @Id AND COMPANY_ID = @company AND LOCATION_ID = @location AND TRIP_NUMBER = @trip AND BAY_ID = @bay";

        private SqlCeCommand cmdInsert;
        private SqlCeCommand cmdUpdate;
        private List<SqlCeParameter> parameters;


        public ProductDaoPDA()
        {
            connectionPDA = new ConnectionPDA();
            connectionPDA.Open();
            
            cmdInsert = new SqlCeCommand(SQL_INSERT_PRODUCT, connectionPDA.connection);
            cmdInsert.Prepare();
            cmdUpdate = new SqlCeCommand(SQL_UPDATE_PALLET, connectionPDA.connection);
            cmdUpdate.Prepare();

            parameters = new List<SqlCeParameter>();
        }

        public ConnectionPDA connectionPDA { get; set; }

        #region IProductDao Members

        /// <summary>
        /// Salva na Base de dados as modificações de um produto.
        /// </summary>
        /// <param name="prod">Produto que será persistido.</param>
        /// <returns>true se produto for salvo corretamente. False caso ocorra     algum erro</returns>
        public bool Update(IProduct prod, IPallet plt)
        {
            Pallet pallet = (Pallet) plt;

            try
            {
                Product product = (Product) prod;

                //cria os parametros que serão usados na clausula Update.
                SqlCeParameter quantity = new SqlCeParameter();
                SqlCeParameter pIdError = new SqlCeParameter();
                SqlCeParameter pId = new SqlCeParameter();
                SqlCeParameter company = new SqlCeParameter();
                SqlCeParameter location = new SqlCeParameter();
                SqlCeParameter descLong = new SqlCeParameter();
                SqlCeParameter descShort = new SqlCeParameter();
                SqlCeParameter picking = new SqlCeParameter();
                SqlCeParameter tripNumber = new SqlCeParameter();
                SqlCeParameter bay = new SqlCeParameter();

                //cria uma lista para armazenar os parametros.
                List<SqlCeParameter> list = new List<SqlCeParameter>();

                //define as propriedades do parametro @iderror.
                pIdError.ParameterName = "iderror";
                pIdError.IsNullable = true;
                if ((product.ErrorType != null) && (product.ErrorType.Id != null))
                    pIdError.Value = product.ErrorType.Id;
                else
                    pIdError.Value = DBNull.Value;

                //define propriedades do parametro @Id
                quantity.ParameterName = "quantity";
                quantity.Value = product.Quantity;

                pId.ParameterName = "Id";
                pId.Value = product.Id;

                descLong.ParameterName = "descLong";
                descLong.Value = prod.DescriptionLong;

                descShort.ParameterName = "descShort";
                descShort.Value = prod.DescriptionShort;

                picking.ParameterName = "picking";
                picking.Value = prod.Picking;

                company.ParameterName = "company";
                company.Value = pallet.CompanyLocation.Company.Id;

                location.ParameterName = "location";
                location.Value = pallet.CompanyLocation.Location.Id;

                tripNumber.ParameterName = "trip";
                tripNumber.Value = pallet.TripNumber;

                bay.ParameterName = "bay";
                bay.Value = pallet.Bay.Id;

                //adiciona parametros à lista
                list.Add(quantity);
                list.Add(pIdError);
                list.Add(pId);
                list.Add(descLong);
                list.Add(descShort);
                list.Add(picking);
                list.Add(company);
                list.Add(location);
                list.Add(tripNumber);
                list.Add(bay);

                //executa o comando

                connectionPDA.ExecutePreparedCommandSql(cmdUpdate, list);

                return true;
            }
            catch (Exception e)
            {
                return false;
            }
        }

        public bool Insert(IProduct product, IPallet plt)
        {
            Pallet pallet = (Pallet) plt;

            try
            {
                //cria parametros.
                SqlCeParameter trip = new SqlCeParameter();
                SqlCeParameter productId = new SqlCeParameter();
                SqlCeParameter bay = new SqlCeParameter();
                SqlCeParameter quantity = new SqlCeParameter();
                SqlCeParameter error = new SqlCeParameter();

                SqlCeParameter company = new SqlCeParameter();
                SqlCeParameter location = new SqlCeParameter();
                SqlCeParameter descLong = new SqlCeParameter();
                SqlCeParameter descShort = new SqlCeParameter();
                SqlCeParameter picking = new SqlCeParameter();

                //define propriedades dos parametros
                trip.ParameterName = "trip";
                trip.Value = pallet.TripNumber;

                productId.ParameterName = "productId";
                productId.Value = product.Id;

                bay.ParameterName = "bay";
                bay.Value = pallet.Bay.Id;

                quantity.ParameterName = "quantity";
                quantity.Value = product.Quantity;

                error.ParameterName = "error";
                if (product.ErrorType.Id == null)
                    error.Value = DBNull.Value;
                else
                    error.Value = product.ErrorType.Id;

                company.ParameterName = "company";
                company.Value = pallet.CompanyLocation.Company.Id;

                location.ParameterName = "location";
                location.Value = pallet.CompanyLocation.Location.Id;

                descLong.ParameterName = "descLong";
                descLong.Value = product.DescriptionLong;

                descShort.ParameterName = "descShort";
                descShort.Value = product.DescriptionShort;

                picking.ParameterName = "picking";
                picking.Value = product.Picking;


                //cria lista e adiciona parametros
                List<SqlCeParameter> list = new List<SqlCeParameter>();
                list.Add(trip);
                list.Add(productId);
                list.Add(bay);
                list.Add(quantity);
                list.Add(error);
                list.Add(company);
                list.Add(location);
                list.Add(descLong);
                list.Add(descShort);
                list.Add(picking);


                connectionPDA.ExecutePreparedCommandSql(cmdInsert, list);

                return true;
            }
            catch (Exception e)
            {
                return false;
            }
        }

        /// <summary>
        /// Obtem uma lista com todos os Produtos de um Pallet.
        /// </summary>
        /// <param name="bayid">bayId do Pallet</param>
        /// <param name="tripNumber">tripNumber do Pallet</param>
        /// <returns>lista com todos os produtos de um Pallet</returns>
        public List<Product> ListProduct(String bayid, int tripNumber, CompanyLocation cp)
        {
            ConnectionPDA con = new ConnectionPDA();

            try
            {
                List<Product> productList = new List<Product>();

                SqlCeParameter parameter = new SqlCeParameter();
                SqlCeParameter param = new SqlCeParameter();
                SqlCeParameter company = new SqlCeParameter();
                SqlCeParameter location = new SqlCeParameter();

                List<SqlCeParameter> list = new List<SqlCeParameter>();

                parameter.ParameterName = "bayid";
                parameter.SqlDbType = SqlDbType.NVarChar;
                parameter.Value = bayid;
                list.Add(parameter);

                param.ParameterName = "trip";
                param.SqlDbType = SqlDbType.Int;
                param.Value = tripNumber;
                list.Add(param);

                company.ParameterName = "company";
                company.Value = cp.Company.Id;
                list.Add(company);

                location.ParameterName = "location";
                location.Value = cp.Location.Id;
                list.Add(location);


                SqlCeDataReader reader = con.Select(SQL_SELECT_PRODUCT, list);

                while (reader.Read())
                {
                    ProductPDA product = new ProductPDA();

                    product.Id = reader.GetString(reader.GetOrdinal("PRODUCT_ID"));

                    product.Quantity = reader.GetInt32(reader.GetOrdinal("QUANTITY"));


                    product.DescriptionLong = reader.GetString(reader.GetOrdinal("PRODUCT_DESCRIPTION_LONG"));
                    product.DescriptionShort = reader.GetString(reader.GetOrdinal("PRODUCT_DESCRIPTION_SHORT"));
                    product.Picking = reader.GetString(reader.GetOrdinal("PICKING_QUANTITY_DESCRIPTION"));


                    productList.Add(product);
                }
                reader.Close();
                return productList;
            }
            finally
            {
                con.Close();
            }
        }

        #endregion
    }
}