﻿#region

using System;
using LoadReview.Persistencia.Dao;
using LoadReviewLibrary.Persistencia.Dao.Generics;
using LoadReviewLibrary.Persistencia.Pojo;
using LoadReviewLibrary.Persistencia.Pojo.Generics;

#endregion

namespace LoadReview.Persistencia.Pojo
{
    [Serializable]
    public class ProductPDA : Product
    {
        #region Delegates

        public delegate int ProductEventHandler();

        public delegate void ProductIncrementHandler(String value);

        public delegate string ProductMaskHandler(int value);

        #endregion

        private String mask;

        private int quantity;

        public ProductPDA()
        {
            Verify = false;
            MontaMascara = MontaMascaraPadrao;
        }

        public String Mask
        {
            get { return mask; }
            set
            {
                if (CheckQuantity(value))
                {
                    mask = value;

                    if (!Verify)
                    {
                        IncrementTot(value);
                        Verify = true;
                    }
                }

                else
                {
                    ErrorType = ErrorTypePDA.Select(SelectError().ToString());
                }
            }
        }

        public override int Quantity
        {
            get { return quantity; }


            set
            {
                mask = MontaMascara(value);
                quantity = value;
            }
        }

        public event ProductEventHandler SelectError;

        public event ProductIncrementHandler IncrementTot;

        public event ProductMaskHandler MontaMascara;


        /// <summary>
        /// Verifica se a Quantidade informada é igual a quantidade definida no objeto.
        /// </summary>
        /// <param name="qtde">Quantidade informada pelo usuário</param>
        /// <returns>true se for igual. False se for diferente.</returns>
        public new bool CheckQuantity(String qtde)
        {
            try
            {
                if (Quantity == Convert.ToInt32(qtde))
                    return true;
            }
            catch
            {
                return false;
            }
            return false;
        }

        public String MontaMascaraPadrao(int value)
        {
            return value.ToString();
        }

        public bool Insert(IPallet pallet, CompanyLocation cp)
        {
            IProductDao productDao = DaoFactory.CreateProductDao();
            return productDao.Insert(this, pallet);
        }
    }
}