﻿#region

using System;
using System.Windows.Forms;
using LoadReview.Persistencia.Dao;
using LoadReview.Persistencia.Pojo;
using LoadReviewLibrary.Persistencia.Dao.Generics;
using LoadReviewLibrary.Persistencia.Pojo;

#endregion

namespace LoadReview
{
    public partial class ReadPallet : Form
    {
        private const int ERROR_ID_PALLET_NULL = 5;

        private const String ERROR_MSG_PALLET_NULL =
            "Não foi encontrado nenhum pallet com o código de barras informado. Atualize os dados de seu PDA.";

        private const String MSG_PALLET_JA_CONFERIDO = "Pallet já foi conferido. Deseja conferir novamente?";

        private CompanyLocation companyLocation;


        private ReadPallet()
        {
            InitializeComponent();
        }

        public ReadPallet(CompanyLocation cp) : this()
        {
            companyLocation = cp;
        }


        /// <summary>
        /// Evento gerado pelo botão de Exit.
        /// Obtem uma instancia da classe saida e mostra 
        /// mensagem de confirmação para o usuário.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnSaida_Click(object sender, EventArgs e)
        {
            //obtem uma instancia estática (singleton)
            Exit saida = Exit.getInstance();
            //mostra msg
            saida.ShowMessageSaida();
        }

        public void LerPallet(int tripNumber, String bayId)
        {
            //obtem o Dao e Cria o Pallet
            IPalletDao daoPallet = DaoFactory.CreatePalletDao();
            PalletPDA pallet = (PalletPDA) daoPallet.Select(companyLocation, bayId, tripNumber);

            if (pallet == null)
                MessageBox.Show(ERROR_MSG_PALLET_NULL, "Pallet não encontrado",
                                MessageBoxButtons.OK,
                                MessageBoxIcon.Asterisk,
                                MessageBoxDefaultButton.Button2);
            else
            {
                DialogResult result = DialogResult.Yes;

                if (pallet.Reviewed)
                {
                    result = MessageBox.Show(MSG_PALLET_JA_CONFERIDO, "Pallet                       conferido",
                                             MessageBoxButtons.YesNo,
                                             MessageBoxIcon.Question,
                                             MessageBoxDefaultButton.Button2);
                }

                if (result == DialogResult.Yes)
                {
                    pallet.CompanyLocation = companyLocation;

                    if (companyLocation.TypeConferencia.Id == 1)
                    {
                        CheckHalfBlind conferencia = new CheckHalfBlind(pallet, companyLocation);
                        conferencia.Show();
                    }
                    else
                    {
                        if (companyLocation.TypeConferencia.Id == 2)
                        {
                            CheckFree conferencia = new CheckFree(pallet, companyLocation);
                            conferencia.Show();
                        }
                    }
                }


                txtBoxPallet.Text = "";
            }
        }

        private void txtBox_KeyPress(object sender, KeyPressEventArgs e)
        {
            //se a tecla <Enter> for pressionada
            if (e.KeyChar == '\r')
            {
                string numeroPallet = txtBoxPallet.Text;

                string bayid = numeroPallet.Substring(6, 2);

                string aux = numeroPallet.Substring(0, (numeroPallet.Length - 2));
                int tripNumber = int.Parse(aux);

                LerPallet(tripNumber, bayid);
            }
        }
    }
}