﻿#region

using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.IO;
using System.Reflection;
using System.Windows.Forms;
using LoadReview.Persistencia.Dao;
using LoadReview.rd_help;
using LoadReview.util;
using LoadReviewLibrary.Persistencia.Pojo;

#endregion

namespace LoadReview.sync
{
    internal class Synchronization
    {
        private const int ERROR_ID_SINCRONIZACAO_BAY = 10;
        private const int ERROR_ID_SINCRONIZACAO_PALLET = 9;
        private const String ERROR_MSG_SINCRONIZACAO_BAY = "Não foi possivel sincronizar todas as Baias";
        private const String ERROR_MSG_SINCRONIZACAO_PALLET = "Não foi possivel sincronizar todos os pallets";


        /// <summary>
        /// Carrega novas informações do banco de dados do Real Distribution.
        /// </summary>
        /// <param name="cp">String com o XML que representa uma CompanyLocation</param>
        /// <returns>true se ocorreu tudo bem.</returns>
        public Boolean SyncPalletRDtoPDA(CompanyLocation cpl)
        {
            RDtoPDA service = new RDtoPDA();
            //localhost.RDtoPDA service = new localhost.RDtoPDA();



            XmlFormater xmlFormaterCpl = new XmlFormater(cpl.GetType());
            XmlFormater xmlFormaterListPallet = new XmlFormater(new List<Pallet>().GetType());

            String name = service.sendPallet(xmlFormaterCpl.Serializar(cpl));

            String url = "http://www.rd-help.com/LoadReview/zip/" + name + ".zip";

            String path = Path.GetDirectoryName(Assembly.GetExecutingAssembly().GetName().CodeBase) + "\\zip\\";
            String fileName = path + name + ".zip";

            
            FileStream fileStream = new FileStream(fileName, FileMode.Create);

            Download download = new Download(url, fileStream);
            if (download.downloadData())
                download.Save();

            SharpZip zip = new SharpZip();
            String listPalletXML = zip.UnzipFile(fileName, path);

            List<Pallet> listPallet = new List<Pallet>();
            XmlFormater xmlFormater = new XmlFormater(listPallet.GetType());


           listPallet = (List<Pallet>) xmlFormater.DesSerializar(listPalletXML);

            Stopwatch marcador = new Stopwatch();
            marcador.Start();
            
            updatePallets(listPallet);

            marcador.Stop();
            String teste = marcador.Elapsed.TotalSeconds.ToString();
            
         return true;
        }


        private Boolean updatePallets(List<Pallet> listPallets)
        {
            int qtdePallets = 0; 
            
            PalletDaoPDA palletDaoPDA = (PalletDaoPDA) DaoFactory.CreatePalletDao();
            foreach (Pallet pallet in listPallets)
            {
                //if (palletDaoPDA.Insert(pallet))
                //    qtdePallets++;

               // else 
                    if (palletDaoPDA.Update(pallet))
                    qtdePallets++;
            }

            
            if (qtdePallets != listPallets.Count)
            {
                MessageBox.Show(ERROR_MSG_SINCRONIZACAO_PALLET, "Sincronizer error", MessageBoxButtons.OK,
                                MessageBoxIcon.Asterisk, MessageBoxDefaultButton.Button3);
                return false;
            }
            return true;
            
        }



        public Boolean SyncBayRDtoPDA(CompanyLocation cpl)
        {
            BayDaoPDA bayDao = (BayDaoPDA) DaoFactory.CreateBayDao();
            try
            {
                int qtdeBays = 0;

                RDtoPDA service = new RDtoPDA();

                List<Bay> listBay = new List<Bay>();
                XmlFormater xmlFormater = new XmlFormater(listBay.GetType());
                String listBayXml = service.sendBay(xmlFormater.Serializar(cpl));



                listBay = (List<Bay>)xmlFormater.DesSerializar(listBayXml);

                foreach (Bay by in listBay)
                {
                    Bay bay = by;


                    if (bayDao.Update(bay, cpl))
                        qtdeBays++;
                    else if (bayDao.Insert(bay, cpl))
                        qtdeBays++;
                }


                if (qtdeBays != listBay.Count)
                {
                    MessageBox.Show(ERROR_MSG_SINCRONIZACAO_BAY, "Sincronizer error", MessageBoxButtons.OK,
                                    MessageBoxIcon.Asterisk, MessageBoxDefaultButton.Button3);
                    return false;
                }
            }
            finally
            {
                bayDao.connectionPDA.Close();
            }

            return true;
        }
    }
}