#region

using System;
using System.Collections.Generic;
using LoadReviewLibrary.Persistencia.Pojo.Generics;

#endregion

namespace LoadReviewLibrary.Persistencia.Pojo
{
    [Serializable]
    public class Pallet : IPallet
    {
        private List<Product> products_;

        public Pallet()
        {
            Reviewed = false;
            products_ = new List<Product>();
        }

        /// <summary>
        /// Construtor padro.
        /// </summary>
        /// <param name="bayId">Bay Id do Pallet que ser criado.</param>
        /// <param name="tripNumber">Trip Number do Pallet a ser criado.</param>
        public Pallet(Bay bayId, int tripNumber) : this()
        {
            TripNumber = tripNumber;
            Bay = bayId;
            products_ = new List<Product>();
        }

        public CompanyLocation CompanyLocation { get; set; }

        public int Total { get; set; }

        #region IPallet Members

        public Bay Bay { get; set; }

        public Vehicle Vehicle { get; set; }

        public int TripNumber { get; set; }

        public DateTime? DeliveryDate { get; set; }

        public int Skus { get; set; }

        public Boolean Reviewed { get; set; }

        public DateTime? StartTime { get; set; }

        public DateTime? EndTime { get; set; }


        /// <summary>
        /// Adiciona uma lista de produtos ao Pallet e incrementa o total de caixas.
        /// </summary>
        public List<Product> Products
        {
            get { return products_; }

            set
            {
                foreach (Product product in value)
                {
                    AddProduct(product);
                }
            }
        }


        public bool Insert()
        {
            throw new NotImplementedException();
        }


        public bool Update()
        {
            throw new NotImplementedException();
        }

        #endregion

        /// <summary>
        /// Adiciona um produto ao Pallet e incrementa o total de caixas do pallet.
        /// </summary>
        /// <param name="product">Produto que ser adicionado.</param>
        public void AddProduct(Product product)
        {
            Total += product.Quantity;
            products_.Add(product);
        }
    }
}