﻿#region

using System;
using System.Collections.Generic;
using System.Data;
using System.Data.SqlClient;

#endregion

namespace LoadReviewSync.persistencia
{
    public class ConnectionSync
    {
        //private static readonly string STRING_CONNECTION = "Data Source=rd-help.com\\SQLEXPRESS;Network Library=DBMSSOCN;Initial Catalog=RSIDistribution2;User ID=loadreview;Password=loadreview;";

       private static readonly string STRING_CONNECTION = "Data Source=C38242-37297\\SQLEXPRESS;Initial Catalog=RSIDistribution2;User ID=loadreview;Password=loadreview;";

       // private static readonly string STRING_CONNECTION = "Data Source=Lucia-PC\\SQLEXPRESS;Initial Catalog=RSIDistribution;User ID=loadreview;Password=loadreview;";



        private SqlTransaction transaction;


        /// <summary>
        /// Estabelece uma conexão com Banco de dados.
        /// </summary>
        /// <param name="str">String de conexão com a Base de dados.</param>
        public ConnectionSync(String str)
        {
            if (connection == null)
                connection = new SqlConnection(str);

            if (connection.State == ConnectionState.Closed)
                connection.Open();
            connection.Close();
        }

        /// <summary>
        /// Construtor padrão. Estabelece uma conexão usando a string de conexão definida na constante
        /// STRINGCONNECTION.
        /// </summary>
        public ConnectionSync() : this(STRING_CONNECTION)
        {
        }

        public SqlConnection connection { get; set; }

        /// <summary>
        /// Abre a conexão com base de dados. 
        /// </summary>
        /// <returns>true se conexão for aberta com sucesso. False se ocorrer alguma          falha</returns>
        public Boolean Open()
        {
            try
            {
                if (connection.State == ConnectionState.Open)
                    return true;
                if (connection.State == ConnectionState.Closed)
                {
                    connection.Open();
                    return true;
                }
                return false;
            }


            catch (Exception e)
            {
                throw new Exception("Ocorreu um problema na conexão dom a base de dados." + e.Message);
            }
        }

        /// <summary>
        /// Fecha a conexão com a base de dados.
        /// </summary>
        /// <returns>True se conexão for finalizada com sucesso. False se ocorrer             alguma falha</returns>
        public Boolean Close()
        {
            try
            {
                if (connection.State == ConnectionState.Open)
                {
                    connection.Close();
                    return true;
                }
            }
            catch (Exception e)
            {
                throw new Exception(
                    "Ocorreu um problema ao encerrar conexão com a base de dados, devido ao seguinte erro:" + e.Message);
            }
            return false;
        }


        /// <summary>
        /// Realiza uma consulta SQL. Fecha a conexao no final.
        /// </summary>
        /// <param name="sql">Comando SELECT que será executado.</param>
        /// <param name="parameters" Lista de parametros que será utilizado na consulta SQL </param>
        /// <returns>DataReader com os dados retornados pela consulta SQL.</returns>
        public SqlDataReader Select(String sql, List<SqlParameter> parameters)
        {
            try
            {
                SqlCommand cmd = new SqlCommand(sql, connection);

                if (connection.State == ConnectionState.Closed)
                    connection.Open();

                //verifica se lista de parametros é válida.
                if ((parameters.Count > 0) && (parameters != null))
                {
                    //adiciona os parametros ao comando.
                    foreach (SqlParameter list in parameters)
                    {
                        cmd.Parameters.AddWithValue(list.ParameterName, list.Value);
                    }
                }

                return cmd.ExecuteReader();
            }
            catch (Exception e)
            {
                throw new Exception("Ocorreu um erro ao consultar BD" + e.Message);
            }
        }


        /// <summary>
        /// Realiza uma consulta SQL. Não fecha a conexão ao final.
        /// </summary>
        /// <param name="cmd">SqlCommand que será executado.</param>
        /// <param name="parameters" Lista de parametros que será utilizado na consulta SQL </param>
        /// <returns>DataReader com os dados retornados pela consulta SQL.</returns>
        public SqlDataReader Select(SqlCommand cmd, List<SqlParameter> parameters)
        {
            try
            {
                if (connection.State == ConnectionState.Closed)
                    connection.Open();

                //verifica se lista de parametros é válida.
                if ((parameters.Count > 0) && (parameters != null))
                {
                    cmd.Parameters.Clear();
                    //adiciona os parametros ao comando.
                    foreach (SqlParameter list in parameters)
                    {
                        cmd.Parameters.AddWithValue(list.ParameterName, list.Value);
                    }
                }

                return cmd.ExecuteReader();
            }
            catch (Exception e)
            {
                throw new Exception("Ocorreu um erro ao consultar BD" + e.Message);
            }
        }


        //////////////// ALTERAÇÃO EXCLUSÃO E INSERÇÃO DE DADOS  ////////////////

        /// <summary>
        /// Executa um comando SQL na base de dados.
        /// </summary>
        /// <param name="sql">Comando SQL que será executado.</param>
        public void ExecuteCommandSql(String sql, List<SqlParameter> parameters)
        {
            try
            {
                SqlCommand cmd = new SqlCommand(sql, connection);


                //verifica se lista de parametros é válida.
                if ((parameters.Count > 0) && (parameters != null))
                {
                    //adiciona os parametros ao comando.
                    foreach (SqlParameter list in parameters)
                    {
                        cmd.Parameters.AddWithValue(list.ParameterName, list.Value);
                    }
                }

                cmd.ExecuteNonQuery();
            }
            catch (Exception e)
            {
                throw new Exception("Ocorreu um problema ao executar o comando:" + sql + "\n Error Message:" +
                                    e.Message);
            }
        }

        //////////////// MÉTODOS PARA TRANSAÇÕES ////////////////

        /// <summary>
        /// inicia uma transação.
        /// </summary>
        public void InitTransaction()
        {
            try
            {
                if (connection.State != ConnectionState.Open)
                    connection.Open();

                transaction = connection.BeginTransaction();
            }
            catch (Exception e)
            {
                throw new Exception("Ocorreu um problema ao inciar a transação devido ao seguinte erro:" +
                                    e.Message);
            }
        }

        /// <summary>
        /// Finaliza uma transação que foi inicia com o método InitTransaction.
        /// </summary>
        /// <param name="flag">true para Commit e false para Rollback.</param>
        public void EndTransaction(Boolean flag)
        {
            if (flag)
                transaction.Commit();
            else
            {
                transaction.Rollback();
            }
        }
    }
}