﻿#region

using System;
using System.Collections.Generic;
using System.Data.SqlClient;
using LoadReviewLibrary.Persistencia.Dao.Generics;
using LoadReviewLibrary.Persistencia.Pojo;

#endregion

namespace LoadReviewSync.persistencia.dao
{
    public class BayDaoSync : IBayDao
    {
        private const String SQL_LIST_BAY =
            "SELECT * FROM T_BAY_MASTER WHERE COMPANY_ID = @company AND LOCATION_ID = @location";

        private const String SQL_SELECT_BAY =
            "SELECT * FROM T_BAY_MASTER WHERE BAY_ID = @Bay AND COMPANY_ID = @Company AND LOCATION_ID = @Location";


        private SqlCommand cmdList;
        private SqlCommand cmdSelect;
        private ConnectionSync connectionSync;
        private List<SqlParameter> parameters;


        public BayDaoSync(ConnectionSync connectionSync)
        {
            this.connectionSync = connectionSync;

            cmdSelect = new SqlCommand(SQL_SELECT_BAY, connectionSync.connection);
            cmdList = new SqlCommand(SQL_LIST_BAY, connectionSync.connection);

            parameters = new List<SqlParameter>();
        }

        #region IBayDao Members

        public Bay Select(string bay, string cp, string locationId)

        {
            parameters.Clear();
            try
            {
                //cria os parametros
                SqlParameter bayid = new SqlParameter();
                SqlParameter company = new SqlParameter();
                SqlParameter location = new SqlParameter();


                //define propriedades dos parametros
                bayid.ParameterName = "Bay";
                bayid.Value = bay;

                company.ParameterName = "Company";
                company.Value = cp;

                location.ParameterName = "Location";
                location.Value = locationId;

                //adiciona os parametros à lista
                parameters.Add(bayid);
                parameters.Add(company);
                parameters.Add(location);

                SqlDataReader reader = connectionSync.Select(cmdSelect, parameters);

                while (reader.Read())
                {
                    Bay baia = new Bay(bay, reader.GetString(1));

                    //libera recursos
                    reader.Close();

                    return baia;
                }


                return null;
            }

            catch (Exception e)
            {
                return null;
            }
        }

        #endregion

        public List<Bay> ListBay(CompanyLocation cpl)
        {
            parameters.Clear();

            SqlDataReader reader;

            List<Bay> listBays = new List<Bay>();

            try
            {
                //cria os paramaetros
                SqlParameter company = new SqlParameter();
                SqlParameter location = new SqlParameter();


                //define propriedades dos parametros
                company.ParameterName = "company";
                company.Value = cpl.Company.Id;

                location.ParameterName = "location";
                location.Value = cpl.Location.Id;

                //adiciona parametros à lista
                parameters.Add(company);
                parameters.Add(location);

                reader = connectionSync.Select(cmdList, parameters);

                while (reader.Read())
                {
                    Bay baia = new Bay(reader.GetString(0), reader.GetString(1));
                    listBays.Add(baia);
                }

                reader.Close();
                return listBays;
            }

            catch (Exception e)
            {
                return null;
            }
        }
    }
}