﻿#region

using System;
using System.Collections.Generic;
using System.Data.SqlClient;
using LoadReviewLibrary.Persistencia.Dao.Generics;
using LoadReviewLibrary.Persistencia.Pojo;
using LoadReviewLibrary.Persistencia.Pojo.Generics;

#endregion

namespace LoadReviewSync.persistencia.dao
{
    public class PalletDaoSync : IPalletDao
    {
        private const String SQL_LIST_PALLET =
            "SELECT p.TRIP_NUMBER, p.BAY_ID, p.PALLET_REVIEWED, p.NUMBER_OF_SKUS, r.START_TIME, r.END_TIME FROM T_LOAD_BAYS p LEFT JOIN T_REVIEWED_PALLET r ON p.LOCATION_ID = r.LOCATION_ID AND p.TRIP_NUMBER = r.TRIP_NUMBER AND p.BAY_ID = r.BAY_ID WHERE p.COMPANY_ID = @company AND p.LOCATION_ID = @location AND p.PALLET_REVIEWED = @reviewed";

        private const String SQL_UPDATE_LOAD_BAYS = "UPDATE T_LOAD_BAYS SET PALLET_REVIEWED = @Reviewed";

        private const String SQL_UPDATE_REVIEWED_PALLET =
            "INSERT INTO T_REVIEWED_PALLET (START_TIME, END_TIME, LOCATION_ID, BAY_ID, TRIP_NUMBER) VALUES(@start, @end, @location, @bay, @trip)";

        #region IPalletDao Members

        /// <summary>
        /// Retorna um Pallet.
        /// </summary>
        /// <param name="cp">CompanyLocation do Pallet a ser retornado.</param>
        /// <param name="bay">BayId do Pallet.</param>
        /// <param name="tripNumber">TripNumber do Pallet.</param>
        /// <returns>Pallet consultado. Retorna null se não for achado nenhum pallet.</returns>
        public IPallet Select(CompanyLocation cp, String bay, int tripNumber)
        {
            throw new Exception("Método não implementado");
        }


        /// <summary>
        /// Obtem uma lista de Pallets.
        /// </summary>
        /// <param name="cp">Company Location dos pallets que serão retornados.</param>
        /// <param name="reviewed">True retorna apenas pallets já conferidos. False retorna apenas              pallets NÃO conferidos</param>
        /// <returns></returns>
        public List<Pallet> PalletList(CompanyLocation cp, Boolean review)
        {
            ConnectionSync connectionSync = new ConnectionSync();
            connectionSync.Open();
            List<Pallet> pallets = new List<Pallet>();
            int count = 0;


            //cria as conexao
            ConnectionSync conProduct = new ConnectionSync();
            ConnectionSync conbay = new ConnectionSync();
            ConnectionSync conVehicle = new ConnectionSync();

            ConnectionSync conError = new ConnectionSync();

            //abre as conexoes
            conProduct.Open();
            conbay.Open();
            conVehicle.Open();
            conError.Open();

            //obtem os Daos
            IProductDao productDao = DaoFactory.CreateProductDao(conProduct, conError);
            IBayDao bayDao = DaoFactory.CreateBayDao(conbay);
            IVehicleDao vehicleDao = DaoFactory.CreateVehicleDao(conVehicle);


            try
            {
                //cria parametros
                SqlParameter company = new SqlParameter();
                SqlParameter location = new SqlParameter();
                SqlParameter reviewed = new SqlParameter();

                //define propriedades dos parametros
                company.ParameterName = "company";
                company.Value = cp.Company.Id;

                location.ParameterName = "location";
                location.Value = cp.Location.Id;

                reviewed.ParameterName = "reviewed";
                reviewed.Value = review ? 1 : 0;
                
                //cria Lista e adiciona Parametros
                List<SqlParameter> list = new List<SqlParameter>();
                list.Add(company);
                list.Add(location);
                list.Add(reviewed);

                //executa consulta SQL
                SqlDataReader reader = connectionSync.Select(SQL_LIST_PALLET, list);

   //varre tod o resultado e guarda os pallets achados dentro do list pallets.
                while (reader.Read())
                {
                    Pallet pallet = new Pallet();
                    pallet.DeliveryDate = null;
                    pallet.StartTime = null;
                    pallet.EndTime = null;

                    pallet.CompanyLocation = cp;

                    pallet.Bay = bayDao.Select(reader.GetString(reader.GetOrdinal("BAY_ID")), cp.Company.Id,
                                               cp.Location.Id);
                    pallet.TripNumber = reader.GetInt32(reader.GetOrdinal("TRIP_NUMBER"));


                    pallet.Reviewed = (bool) reader.GetValue(reader.GetOrdinal("PALLET_REVIEWED"));
                    
                    if (reader.GetValue(reader.GetOrdinal("START_TIME")) != DBNull.Value)
                        pallet.StartTime = (DateTime) reader.GetValue(reader.GetOrdinal("START_TIME"));

                    if (reader.GetValue(reader.GetOrdinal("END_TIME")) != DBNull.Value)
                        pallet.EndTime = (DateTime) reader.GetValue(reader.GetOrdinal("END_TIME"));

                    if (reader.GetValue(reader.GetOrdinal("NUMBER_OF_SKUS")) != DBNull.Value)
                    pallet.Skus = reader.GetInt32(reader.GetOrdinal("NUMBER_OF_SKUS"));

                    pallet.Vehicle = new Vehicle("1", "demo");
                    //pallet.Vehicle = vehicleDao.Select(cp.Company.Id, cp.Location.Id,
                          //                             reader.GetInt32(reader.GetOrdinal("TRIP_NUMBER")));


                    List<Product> products = productDao.ListProduct(pallet.Bay.Id, pallet.TripNumber, cp);


                    pallet.Products = products;

                    pallets.Add(pallet);
                }
                reader.Close();
                return pallets;
            }


            catch (Exception e)
            {
                throw new Exception(e.Message);
            }
            finally
            {
                connectionSync.Close();
                conProduct.Close();
                conbay.Close();
                conVehicle.Close();
                conError.Close();
            }
        }


        public bool Insert(IPallet pallet)
        {
            throw new NotImplementedException();
        }

        public Boolean Update(IPallet plt)
        {
            ConnectionSync connectionSync = new ConnectionSync();
            ConnectionSync conProd = new ConnectionSync();
            conProd.Open();

            Pallet pallet = (Pallet) plt;
            connectionSync.Open();

            try

            {
                //Cria os parametros

                //parametro reviewed - usado no UPDATE na tabela T_LOAD_BAYS
                SqlParameter reviewed = new SqlParameter();

                //PARAMETROS USADOS NO UPDATE DA TABELA T_REVIEWED_PALLET
                SqlParameter start = new SqlParameter();
                SqlParameter end = new SqlParameter();
                SqlParameter location = new SqlParameter();
                SqlParameter bay = new SqlParameter();
                SqlParameter trip = new SqlParameter();

                //Cria lista para guardar parametros
                List<SqlParameter> parametersLoadBays = new List<SqlParameter>();

                //lista usada no update da tabela T_REVIEWED_PALLET
                List<SqlParameter> parametersReviewed = new List<SqlParameter>();

                //define propriedades dos parametros
                reviewed.ParameterName = "Reviewed";
                reviewed.Value = pallet.Reviewed;

                start.ParameterName = "start";
                start.Value = pallet.StartTime;

                end.ParameterName = "end";
                end.Value = pallet.EndTime;

                location.ParameterName = "location";
                location.Value = pallet.CompanyLocation.Location.Id;

                bay.ParameterName = "bay";
                bay.Value = pallet.Bay.Id;

                trip.ParameterName = "tripNumber";
                trip.Value = pallet.TripNumber;

                //adiciona os parametros à lista
                parametersLoadBays.Add(reviewed);
                parametersReviewed.Add(start);
                parametersReviewed.Add(end);
                parametersReviewed.Add(location);
                parametersReviewed.Add(bay);
                parametersReviewed.Add(trip);

                //inicia uma transação
                connectionSync.InitTransaction();

                //executa os comandos.
                connectionSync.ExecuteCommandSql(SQL_UPDATE_LOAD_BAYS, parametersLoadBays);
                connectionSync.ExecuteCommandSql(SQL_UPDATE_REVIEWED_PALLET, parametersReviewed);

                //cria o dao para atualização de Product
                IProductDao dao = DaoFactory.CreateProductDao(conProd, null);

                //para cada produto da lista, executa o update  
                foreach (Product prod in pallet.Products)
                {
                    dao.Update(prod, pallet);
                }

                //encerra a transação com sucesso
                connectionSync.EndTransaction(true);
            }
            catch (Exception e)
            {
                connectionSync.EndTransaction(false);
                throw new Exception(e.Message);
            }

            finally
            {
                conProd.Close();
            }


            return false;
        }

        #endregion
    }
}