﻿#region

using System;
using System.Collections.Generic;
using System.Data.SqlClient;
using LoadReviewLibrary.Persistencia.Dao.Generics;
using LoadReviewLibrary.Persistencia.Pojo;
using LoadReviewLibrary.Persistencia.Pojo.Generics;

#endregion

namespace LoadReviewSync.persistencia.dao
{
    public class ProductDaoSync : IProductDao
    {
        private const string SQL_LIST_PRODUCT =
            "SELECT p.PRODUCT_ID, p.QUANTITY, p.ERROR_ID, m.PICKING_QUANTITY_DESCRIPTION, m.PRODUCT_DESCRIPTION_LONG, m.PRODUCT_DESCRIPTION_SHORT FROM T_LOAD_PALLET_PRODUCT_SUMMARY p LEFT JOIN T_PRODUCT_MASTER m ON p.COMPANY_ID = m.COMPANY_ID AND p.LOCATION_ID = m.LOCATION_ID AND p.PRODUCT_ID = m.PRODUCT_ID WHERE p.COMPANY_ID = @company AND p.LOCATION_ID = @location AND TRIP_NUMBER = @trip AND BAY_ID = @bay";

        private const string SQL_UPDATE_PRODUCT =
            "UPDATE LOAD_PALLET_PRODUCT_SUMMARY SET ERROR_ID = @error WHERE LOCATION_ID = @location AND BAY_ID = @bay AND TRIP_NUMBER = @trip AND PRODUCT_ID = @productId";


        //atributos de classe por questões de PERFORMANCE.

        private SqlCommand cmdListProduct;
        private SqlCommand cmdUpdate;
        private ConnectionSync conError;

        private List<SqlParameter> parameters;


        public ProductDaoSync(ConnectionSync connectionSync, ConnectionSync conError)
        {
            this.connectionSync = connectionSync;
            this.conError = conError;
            cmdUpdate = new SqlCommand(SQL_UPDATE_PRODUCT, connectionSync.connection);
            cmdListProduct = new SqlCommand(SQL_LIST_PRODUCT, connectionSync.connection);

            parameters = new List<SqlParameter>();
        }

        private ConnectionSync connectionSync { get; set; }

        #region IProductDao Members

        public Boolean Update(IProduct product, IPallet plt)
        {
            parameters.Clear();

            Pallet pallet = (Pallet) plt;

            try
            {
                //Cria os parametros
                SqlParameter error = new SqlParameter();
                SqlParameter location = new SqlParameter();
                SqlParameter bay = new SqlParameter();
                SqlParameter trip = new SqlParameter();
                SqlParameter productId = new SqlParameter();

                //Define propriedades dos parametros
                error.ParameterName = "error";
                error.Value = product.ErrorType.Id;

                location.ParameterName = "location";
                location.Value = pallet.CompanyLocation.Location.Id;

                bay.ParameterName = "bay";
                bay.Value = pallet.Bay.Id;

                trip.ParameterName = "trip";
                trip.Value = pallet.TripNumber;

                productId.ParameterName = "productId";
                productId.Value = product.Id;

                parameters.Add(error);
                parameters.Add(location);
                parameters.Add(bay);
                parameters.Add(trip);
                parameters.Add(productId);

                connectionSync.ExecuteCommandSql(SQL_UPDATE_PRODUCT, parameters);

                return true;
            }

            catch (SqlException e)
            {
                throw new Exception("Ocorreu um erro ao salvar Produto" + e.Message);
            }
        }

        public List<Product> ListProduct(string bayId, int tripNumber, CompanyLocation cp)
        {
            parameters.Clear();
            int count = 0;
            try
            {
                List<Product> produtos = new List<Product>();

                //cria os parametros
                SqlParameter company = new SqlParameter();
                SqlParameter location = new SqlParameter();
                SqlParameter trip = new SqlParameter();
                SqlParameter bay = new SqlParameter();

                //define propriedades dos parametros
                company.ParameterName = "company";
                company.Value = cp.Company.Id;

                location.ParameterName = "location";
                location.Value = cp.Location.Id;

                trip.ParameterName = "trip";
                trip.Value = tripNumber;

                bay.ParameterName = "bay";
                bay.Value = bayId;

                //cria a Lista e adiciona os parametros.
                parameters.Add(company);
                parameters.Add(location);
                parameters.Add(trip);
                parameters.Add(bay);

                //obtem o Dao de ErrorType
                IErrorTypeDao errorTypeDao = DaoFactory.CreateErrorTypeDao(conError);

                SqlDataReader reader = connectionSync.Select(cmdListProduct, parameters);


                
                while (reader.Read())
                {
                    count++;
                    if (count == 13)
                    {
                        String parar;
                    }
                    Product product = new Product();
                    product.ErrorType = null;

                    product.Id = reader.GetString(reader.GetOrdinal("PRODUCT_ID"));
                    product.Quantity = (int) reader.GetFloat(reader.GetOrdinal("QUANTITY"));
                    product.Picking = reader.GetString(reader.GetOrdinal("PICKING_QUANTITY_DESCRIPTION"));
                    product.DescriptionLong = reader.GetString(reader.GetOrdinal("PRODUCT_DESCRIPTION_LONG"));
                    product.DescriptionShort = reader.GetString(reader.GetOrdinal("PRODUCT_DESCRIPTION_SHORT"));

                    if (reader.GetValue(reader.GetOrdinal("ERROR_ID")) != DBNull.Value)
                        product.ErrorType = errorTypeDao.Select(reader.GetString(reader.GetOrdinal("ERROR_ID")));


                    produtos.Add(product);
                }
                reader.Close();
                return produtos;
            }
            catch (Exception e)
            {
                
                throw new Exception(e.Message);
            }

        }

        public bool Insert(IProduct product, IPallet pallet)
        {
            throw new NotImplementedException();
        }

        #endregion
    }
}