﻿using System;
using System.Collections.Generic;
using System.Data.SqlClient;
using System.Linq;
using System.Web;
using LoadReviewLibrary.Persistencia.Pojo;

namespace LoadReviewSync.persistencia.dao
    {
    public class SyncDao
        {

        private const String SQL_LIST_TRIPNUMBER =
            "SELECT distinct p.TRIP_NUMBER FROM T_LOAD_BAYS p LEFT JOIN T_REVIEWED_PALLET r ON p.LOCATION_ID = r.LOCATION_ID AND p.TRIP_NUMBER = r.TRIP_NUMBER AND p.BAY_ID = r.BAY_ID WHERE p.COMPANY_ID = @company AND p.LOCATION_ID = @location AND p.PALLET_REVIEWED = @reviewed";




        /// <summary>
        /// Obtem uma lista de TripNumbers à serem carregados.
        /// </summary>
        /// <param name="cp">Company Location dos pallets que serão retornados.</param>
        /// <param name="reviewed">True retorna apenas pallets já conferidos. False retorna apenas              pallets NÃO conferidos</param>
        /// <returns></returns>
        public List<int> TripNumberList(CompanyLocation cp, Boolean review)
            {
            ConnectionSync connectionSync = new ConnectionSync();
            connectionSync.Open();
            List<int> trips = new List<int>();
            int count = 0;
            
            try
                {
                //cria parametros
                SqlParameter company = new SqlParameter();
                SqlParameter location = new SqlParameter();
                SqlParameter reviewed = new SqlParameter();

                //define propriedades dos parametros
                company.ParameterName = "company";
                company.Value = cp.Company.Id;

                location.ParameterName = "location";
                location.Value = cp.Location.Id;

                reviewed.ParameterName = "reviewed";
                reviewed.Value = review ? 1 : 0;

                //cria Lista e adiciona Parametros
                List<SqlParameter> list = new List<SqlParameter>();
                list.Add(company);
                list.Add(location);
                list.Add(reviewed);

                //executa consulta SQL
                SqlDataReader reader = connectionSync.Select(SQL_LIST_TRIPNUMBER, list);

                //varre tod o resultado e guarda os pallets achados dentro do list pallets.
                while (reader.Read())
                    {
                    trips.Add(reader.GetInt32(reader.GetOrdinal("TRIP_NUMBER")));
                    }
               
                
                reader.Close();
                return trips;
                }


            catch (Exception e)
                {
                throw new Exception(e.Message);
                }
            finally
                {
                connectionSync.Close();
                }
            }




        }
    }
