﻿#region

using System;
using System.IO;
using ICSharpCode.SharpZipLib.Zip;

#endregion

namespace LoadReviewSync.util
{
    public class SharpZip
    {


        /// <summary>
        /// Creates a zip file
        /// </summary>
        /// <param name="zipFileStoragePath">where to store the zip file</param>
        /// <param name="zipFileName">the zip file filename</param>
        /// <param name="fileSystemInfoToZip">the directory/file to zip</param>
        /// <returns>indicates whether the file was created successfully</returns>
        public bool CreateZipFile(string zipFileName
                                   , FileSystemInfo fileSystemInfoToZip)
        {
            return CreateZipFile(zipFileName
                                 , new[]
                                       {
                                           fileSystemInfoToZip
                                       });
        }



        /// <summary>
        /// A function that creates a zip file
        /// </summary>
        /// <param name="zipFileStoragePath">location where the file should be created</param>
        /// <param name="zipFileName">the filename of the zip file</param>
        /// <param name="fileSystemInfosToZip">an array of filesysteminfos that needs to be added to the file</param>
        /// <returns>a bool value that indicates whether the file was created</returns>
        public bool CreateZipFile(string zipFileName
                                   , FileSystemInfo[] fileSystemInfosToZip)
        {
            // a bool variable that says whether or not the file was created
            bool isCreated = false;

            try
            {
                //create our zip file
                ZipFile z = ZipFile.Create(zipFileName);
          
                //initialize the file so that it can accept updates
                z.BeginUpdate();
                //get all the files and directory to zip
               
                GetFilesToZip(fileSystemInfosToZip, z);
                //commit the update once we are done
                z.CommitUpdate();
                //close the file
                z.Close();
                //success!
                isCreated = true;
            }
            catch (Exception ex)
            {
                //failed
                isCreated = false;
                //lets throw our error
                throw new Exception("Erro ao criar arquivo Zip." + ex.Message);
            }

            //return the creation status
            return isCreated;
        }

        /// <summary>
        /// Iterate thru all the filesysteminfo objects and add it to our zip file
        /// </summary>
        /// <param name="fileSystemInfosToZip">a collection of files/directores</param>
        /// <param name="z">our existing ZipFile object</param>
        public void GetFilesToZip(FileSystemInfo[] fileSystemInfosToZip, ZipFile z)
        {
            //check whether the objects are null
            if (fileSystemInfosToZip != null && z != null)
            {
                //iterate thru all the filesystem info objects
                foreach (FileSystemInfo fi in fileSystemInfosToZip)
                {
                    //check if it is a directory
                    if (fi is DirectoryInfo)
                    {
                        DirectoryInfo di = (DirectoryInfo) fi;
                        //add the directory
                        z.AddDirectory(di.FullName);
                        //drill thru the directory to get all
                        //the files and folders inside it.
                       GetFilesToZip(di.GetFileSystemInfos(), z);
                    }
                    else
                    {
                        //add it
                       
                        z.Add(fi.FullName, fi.Name);
                       
                    }
                }
            }
        }

        public void UnzipFile(String file)
        {

        if (!File.Exists(file))
            {
            Console.WriteLine("Cannot find file '{0}'", file);
            return;
            }

        using (ZipInputStream s = new ZipInputStream(File.OpenRead(file)))
            {

            ZipEntry theEntry;
 
            while ((theEntry = s.GetNextEntry()) != null)
                {

                Console.WriteLine(theEntry.Name);

                string directoryName = Path.GetDirectoryName(theEntry.Name);
                string fileName = Path.GetFileName(theEntry.Name);

                // create directory
                if (directoryName.Length > 0)
                    {
                    Directory.CreateDirectory(directoryName);
                    }

                if (fileName != String.Empty)
                    {
                    using (FileStream streamWriter = File.Create(theEntry.Name))
                        {

                        int size = 2048;
                        byte[] data = new byte[2048];
                        while (true)
                            {
                            size = s.Read(data, 0, data.Length);
                            if (size > 0)
                                {
                                streamWriter.Write(data, 0, size);
                                }
                            else
                                {
                                break;
                                }
                            }
                        }
                    }
                }
            }



        }
    }

    }